

Imports Cryptware.NCryptoki
Imports System.Security.Cryptography.X509Certificates
Imports System.Security.Cryptography
Imports System.Text

Module Module1


    Sub Main()
        Dim cryptoki As Cryptoki
        Dim info As CryptokiInfo
        Dim tokenslots As SlotList
        Dim token As Token
        Dim session As Session
        Dim res As Integer
        Dim template As CryptokiCollection
        Dim objects As CryptokiCollection
        Dim obj As Object
        Dim i As Integer

        ' Creates a Cryptoki object related to the specific PKCS#11 native library 
        cryptoki = New Cryptoki("smaoscki.dll")

        cryptoki.Initialize()

        'Prints all information relating to the native library
        info = cryptoki.Info
        Console.WriteLine(info.Version)
        Console.WriteLine(info.ManufacturerID)
        Console.WriteLine(info.LibDescription)

        ' Reads the set of slots containing a token
        tokenslots = cryptoki.ActiveSlots
        If (tokenslots.Count = 0) Then Throw New Exception("No token inserted")

        ' Gets the first token available
        token = tokenslots(0).Token

        ' Opens a read/write serial session
        session = token.OpenSession((Session.CKF_SERIAL_SESSION or Session.CKF_RW_SESSION), nothing,nothing)

        ' Executes the login passing the user PIN
        res = session.Login(session.CKU_USER, "12345678")
        If (res <> 0) Then
            Console.WriteLine("Wrong PIN")
            Exit Sub
        End If

        Console.WriteLine("Logged in:" + session.IsLoggedIn)

        ' Searchs for an RSA private key object
        ' Sets the template with its attributes
        template = New CryptokiCollection()
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_CLASS, CryptokiObject.CKO_PRIVATE_KEY))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_KEY_TYPE, Key.CKK_RSA))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_LABEL, "Ugo's new Key 0"))

        ' Launchs the search specifying the template just created
        objects = session.Objects.Find(template, 10)

        For Each obj In objects
            Console.WriteLine(CType(obj, PrivateKey).Label)
        Next

        For i = 0 To objects.Count - 1
            Console.WriteLine(CType(objects(i), PrivateKey).Label)
        Next

        Dim privateKey As RSAPrivateKey
        Dim publicKey As RSAPublicKey

        ' If the private keys is found computes the digital signature 
        If (objects.Count = 0) Then
            Dim templatePub As CryptokiCollection
            Dim templatePri As CryptokiCollection

            templatePub = New CryptokiCollection()
            templatePub.Add(New ObjectAttribute(ObjectAttribute.CKA_CLASS, CryptokiObject.CKO_PUBLIC_KEY))
            templatePub.Add(New ObjectAttribute(ObjectAttribute.CKA_TOKEN, True))
            templatePub.Add(New ObjectAttribute(ObjectAttribute.CKA_PRIVATE, False))
            templatePub.Add(New ObjectAttribute(ObjectAttribute.CKA_LABEL, "Ugo's new Key"))
            templatePub.Add(New ObjectAttribute(ObjectAttribute.CKA_ID, "1"))
            templatePub.Add(New ObjectAttribute(ObjectAttribute.CKA_MODULUS_BITS, 1024))
            templatePub.Add(New ObjectAttribute(ObjectAttribute.CKA_PUBLIC_EXPONENT, &H10001))

            templatePri = New CryptokiCollection()
            templatePri.Add(New ObjectAttribute(ObjectAttribute.CKA_CLASS, CryptokiObject.CKO_PRIVATE_KEY))
            templatePri.Add(New ObjectAttribute(ObjectAttribute.CKA_TOKEN, True))
            templatePri.Add(New ObjectAttribute(ObjectAttribute.CKA_PRIVATE, True))
            templatePri.Add(New ObjectAttribute(ObjectAttribute.CKA_LABEL, "Ugo's new Key 0"))
            templatePri.Add(New ObjectAttribute(ObjectAttribute.CKA_ID, "1"))


            'gets the first object
            Dim keys As Key()
            keys = session.GenerateKeyPair(Mechanism.RSA_PKCS_KEY_PAIR_GEN, templatePub, templatePri)
            privateKey = CType(keys(1), RSAPrivateKey)
            publicKey = CType(keys(0), RSAPublicKey)
        Else

            privateKey = CType(objects(objects.Count - 1), RSAPrivateKey)
            Console.WriteLine(privateKey.Label)

            template = New CryptokiCollection()
            template.Add(New ObjectAttribute(ObjectAttribute.CKA_CLASS, CryptokiObject.CKO_PUBLIC_KEY))
            template.Add(New ObjectAttribute(ObjectAttribute.CKA_LABEL, "Ugo's new Key 0"))
            template.Add(New ObjectAttribute(ObjectAttribute.CKA_ID, "1"))

            ' Launchs the search specifying the template just created  
            objects = session.Objects.Find(template, 1)

            publicKey = CType(objects(0), RSAPublicKey)
            Dim text As Byte()
            Dim signature As Byte()

            text = Encoding.ASCII.GetBytes("Hello World")


            ' launches the digital signing operation with a RSA_PKCS mechanism
            res = session.SignInit(Mechanism.SHA1_RSA_PKCS, privateKey)

            ' computes the signature
            signature = session.Sign(text)

            Console.Write(signature)

            res = session.VerifyInit(Mechanism.SHA1_RSA_PKCS, publicKey)

            res = session.Verify(text, signature)

            Console.Write("Verify " & res)
        End If

        ' Logouts and closes the session
        session.Logout()
        session.Close()
        cryptoki.Finalize(IntPtr.Zero)
    
    End Sub

    Function ImportCertificate(ByVal session As Session, ByVal cert As X509Certificate2, ByVal id As String, ByVal label As String, ByVal modifiable As Boolean) As CryptokiObject

        Dim template As CryptokiCollection
        Dim certif As Certificate

        template = New CryptokiCollection()
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_CLASS, CryptokiObject.CKO_CERTIFICATE))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_SUBJECT, cert.SubjectName.RawData))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_ISSUER, cert.Issuer))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_SERIAL_NUMBER, cert.SerialNumber))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_ID, id))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_LABEL, label))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_TOKEN, True))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_VALUE, cert.RawData))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_MODIFIABLE, modifiable))

        certif = session.Objects.Create(template)

        ImportCertificate = certif

    End Function

    Function ImportKeyPair(ByVal Session As Session, ByVal cert As Byte(), ByVal password As String, ByVal id As String, ByVal label As String, ByVal priv As Boolean, ByVal modifiable As Boolean) As PrivateKey

        Dim certificate As X509Certificate2
        certificate = New X509Certificate2(cert, password, X509KeyStorageFlags.Exportable)

        If (Not certificate.HasPrivateKey) Then
            Throw New Exception("Certificate doesn't have private key. Import failed!")
        End If


        Dim keyPair As RSA
        Dim keyParams As RSAParameters
        Dim template As CryptokiCollection
        Dim priKey As PrivateKey

        keyPair = certificate.PrivateKey

        keyParams = keyPair.ExportParameters(True)
        template = New CryptokiCollection()
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_CLASS, CryptokiObject.CKO_PRIVATE_KEY))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_SUBJECT, certificate.SubjectName.RawData))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_ID, id))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_LABEL, label))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_TOKEN, True))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_MODULUS, keyParams.Modulus))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_PUBLIC_EXPONENT, keyParams.Exponent))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_PRIVATE_EXPONENT, keyParams.D))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_PRIVATE, priv))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_MODIFIABLE, modifiable))
        priKey = Session.Objects.Create(template)


        ImportKeyPair = priKey
    End Function

    Function ImportData(ByVal session As Session, ByVal databuffer As Byte(), ByVal label As String, ByVal app As String, ByVal priv As Boolean, ByVal modifiable As Boolean) As Data

        Dim template As CryptokiCollection
        Dim data As Data

        template = New CryptokiCollection()
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_CLASS, CryptokiObject.CKO_DATA))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_LABEL, label))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_APPLICATION, app))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_TOKEN, True))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_PRIVATE, priv))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_MODIFIABLE, modifiable))
        template.Add(New ObjectAttribute(ObjectAttribute.CKA_VALUE, databuffer))

        data = CType(session.Objects.Create(template), Data)

        ImportData = data
    End Function
End Module
